
#pragma once

#define VSTGUI_ENABLE_DEPRECATED_METHODS 0

#include "public.sdk/source/vst/vstaudioeffect.h"
#include "QCue.hpp"

#define VSTGUI_LIVE_EDITING 0

namespace ShadowTec {
	class CQSocketUDP;
}

// **************************************************************************
namespace Steinberg {
namespace Vst {

class IParamValueQueue;

/// <summary>The <b>EOSXsavestate</b> is the import/export structure for
/// preference settings.</summary>
struct EOSXsavestate {
	/// <summary>flag-reserved bypass-mode switch</summary>
	int32 bypass;
	/// <summary>port we're connecting to</summary>
	uint16 nPort;
	/// <summary>current cue we're executing</summary>
	QCue currentCue;
};


// **************************************************************************
/// <summary><b>EOSX:</b>main plugin-class for the ShadowTec EOSX control
/// connector VST plug-in.</summary>
class EOSX : public AudioEffect
{
public:
	//-----------------------------------------------------------------------
	/// <section>constructors and destructors</section>

	/// <summary>generic constructor</summary>
	EOSX();

	/// <summary>virtual destructor</summary>
	virtual ~EOSX();

	//-----------------------------------------------------------------------
	/// <section>VST initialization</section>

	/// <summary>From <b>FUnknown</b>: get interface-pointer by iid</summary>
	virtual tresult PLUGIN_API queryInterface(const TUID _iid, void** obj)
		SMTG_OVERRIDE;

	/// <summary>From <b>AudioEffect</b>: Called at first after constructor
	tresult PLUGIN_API initialize(FUnknown* context) SMTG_OVERRIDE;

	//-----------------------------------------------------------------------
	/// <section>Automatization and serialization</section>

	/// <summary>From <b>AudioEffect</b>: load model from bitstream</summary>
	tresult PLUGIN_API setState (IBStream* state) SMTG_OVERRIDE;

	/// <summary>From <b>AudioEffect</b>: safe model into bitstream</summary>
	tresult PLUGIN_API getState (IBStream* state) SMTG_OVERRIDE;

	/// <summary>From <b>AudioEffect</b>: automate parameter change</summary>
	tresult PLUGIN_API process(ProcessData& data) SMTG_OVERRIDE;

	/// <summary>apply parameter change</summary>
	tresult ParamChangeRead(IParamValueQueue* paramQueue);

	//-----------------------------------------------------------------------
	/// <section>static class functions and fields</section>

	/// <summary>This plug-ins GUIDv6.5/ShTc-format for the framework and
	/// its factories.</summary>
	static FUID cid;

	/// <summary>create function required for Plug-in factory, it will be
	/// called to create new instances of this Plug-in</summary>
	static FUnknown* createInstance(void* context)
		{ return (IAudioProcessor*) new EOSX; }

	/// <summary>export values into stream</summary>
	static tresult PLUGIN_API exportSaveState(IBStream* state,
		EOSXsavestate const& rState);

	/// <summary>import values from stream</summary>
	static tresult PLUGIN_API importSaveState(IBStream* state,
		EOSXsavestate& rState);

	//-----------------------------------------------------------------------
protected:
	/// <summary>pointer to socket connector</summary>
	ShadowTec::CQSocketUDP* mp_Socket;
	/// <summary>our savestate and context</summary>
	EOSXsavestate saveState;
	/// <summary>flag set if we're in bypass mode</summary>
	bool bBypass;
	/// <summary>flag determing our connection mode to eos</summary>
	bool bConnected;
};

// **************************************************************************
}} // namespaces
