
#include "EOSXcontroller.h"
#include "EOSXparamids.h"
#include "pluginterfaces/base/futils.h"
#include "QIPControl.h"

using namespace VSTGUI;

// **************************************************************************
namespace Steinberg {
namespace Vst {

// controllers GUIDv6.5/ShTc-format for the framework and its factories.
FUID EOSXController::cid(0x53686164, 0x6F775465, 0x63000001, 0x454F5358);

IMPLEMENT_REFCOUNT(EOSXController)


// **************************************************************************
// VST initialization
// **************************************************************************

// --------------------------------------------------------------------------
tresult PLUGIN_API EOSXController::queryInterface(const TUID id, void** obj)
{
	QUERY_INTERFACE(id, obj, EOSXController::cid, EOSXController)
	return EditController::queryInterface(id, obj);
}

// --------------------------------------------------------------------------
// called at first after constructor
tresult PLUGIN_API EOSXController::initialize (FUnknown* context)
{
	tresult result = EditController::initialize (context);
	if (result != kResultOk)
		return result;

	CIPControlCreator factory;

	// ---Create Parameters------------
	parameters.addParameter(
		String ("Bypass"), 0, 1, 0,
		ParameterInfo::kCanAutomate | ParameterInfo::kIsBypass, kBypassId
	);
	parameters.addParameter(new RangeParameter(
		STR16("CueList"), kCueListId, nullptr, 1, 999, 1, 998,
		ParameterInfo::kCanAutomate, kRootUnitId
	));
	parameters.addParameter(new RangeParameter(
		STR16("Cue"), kCueId, nullptr, 0, 999, 1, 999,
		ParameterInfo::kCanAutomate, kRootUnitId
	));
	parameters.addParameter(new RangeParameter(
		STR16("SubCue"), kSubCueId, nullptr, 0, 999, 1, 999,
		ParameterInfo::kCanAutomate, kRootUnitId
	));

	return result;
}

// --------------------------------------------------------------------------
// create a new view
IPlugView* PLUGIN_API EOSXController::createView(FIDString name)
{
	if (ConstString(name) == ViewType::kEditor)
	{
		return new VST3Editor(this, "EOSXEditor", "EOSX.uidesc");
	}
	return nullptr;
}

// --------------------------------------------------------------------------
// set parameter from value
tresult PLUGIN_API EOSXController::setParamNormalized (ParamID tag,
	ParamValue value)
{
	tresult res = EditController::setParamNormalized (tag, value);
	if (res != kResultOk)
		return res;

	switch (tag)
	{
	case kCueListId:
		saveState.currentCue.list = value;
		break;
	case kCueId:
		saveState.currentCue.cue = value;
		break;
	case kSubCueId:
		saveState.currentCue.subcue = value;
		break;
	}

	return res;
}

// --------------------------------------------------------------------------
// we receive the current state of the component (processor part)
tresult PLUGIN_API EOSXController::setComponentState (IBStream* state)
{
	if (!state)
		return kResultFalse;

	// read savestate from stream
	EOSX::importSaveState(state, saveState);

	// set parameters from saved values
	EditController::setParamNormalized( kCueListId,
		ToNormalized<ParamValue> (saveState.currentCue.list, 999)
	);
	EditController::setParamNormalized( kCueId,
		ToNormalized<ParamValue>(saveState.currentCue.cue, 999)
	);
	EditController::setParamNormalized( kSubCueId,
		ToNormalized<ParamValue>(saveState.currentCue.subcue, 999)
	);

	return kResultOk;
}

// --------------------------------------------------------------------------
// The value of a control changed
void EOSXController::valueChanged(CControl* pControl)
{
	COnOffButton* pBtn = dynamic_cast<COnOffButton*>(pControl);
	if (pBtn == nullptr)
		return;
	pBtn->setBackground(pBtn->getDisabledBackground());
	pBtn->setDirty(true);
}

void MyUIController::valueChanged(CControl* pControl)
{
	COnOffButton* pBtn = dynamic_cast<COnOffButton*>(pControl);
	if (pBtn == nullptr)
		return;
	pBtn->setBackground(pBtn->getDisabledBackground());
	pBtn->setDirty(true);
}


// **************************************************************************
}} // namespaces
