#pragma once

#include "public.sdk/source/vst/vsteditcontroller.h"
#include "vstgui/plugin-bindings/vst3editor.h"
#include "EOSX.h"

#include "vstgui/plugin-bindings/vst3padcontroller.h"


// **************************************************************************
namespace Steinberg {
namespace Vst {



// **************************************************************************
// test suggestion from sdk.steinberg.net
class MyUIController : public DelegationController
{
public:
	MyUIController(IController* parent) : DelegationController(parent) {}
	IControlListener* getControlListener(UTF8StringPtr controlTagName)
		override { return this; }
	void valueChanged(CControl* pControl);
};


// **************************************************************************
/// <summary><b>EOSXController</b>: GUI-based controller interface fot the
/// EOSX connector plug-in.<summary>
class EOSXController : public EditController, VSTGUI::VST3EditorDelegate,
	IController
{
public:
	//-----------------------------------------------------------------------
	/// <section>constructors and destructors</section>

	/// <summary>generic constructor</summary>
	EOSXController() {};

	/// <summary>virtual destructor</summary>
	virtual ~EOSXController() {};

	//-----------------------------------------------------------------------
	/// <section>static class functions and fields</section>

	/// <summary>This controllers GUIDv6.5/ShTc-format for the framework and
	/// its factories.</summary>
	static FUID cid;

	/// <summary>create function required for Plug-in factory, it will be
	/// called to create new instances of this Plug-in</summary>
	static FUnknown* createInstance(void* context)
		{ return (IEditController*) new EOSXController(); }

	//-----------------------------------------------------------------------
	/// <section>VST initialization</section>
	DECLARE_FUNKNOWN_METHODS

	/// <summary>From <b>IPluginBase</b>: Called at first after constructor
	tresult PLUGIN_API initialize(FUnknown* context) SMTG_OVERRIDE;

	/// <summary>From <b>EditController</b>: create new view
	virtual IPlugView* PLUGIN_API createView(FIDString name) SMTG_OVERRIDE;

	/// <summary>From <b>EditController</b>: set parameter from value
	tresult PLUGIN_API setParamNormalized(ParamID tag, ParamValue value)
		SMTG_OVERRIDE;

	/// <summary>From <b>EditController</b>: we receive the current state of
	/// the component (processor part)</summary>
	tresult PLUGIN_API setComponentState(IBStream* state) SMTG_OVERRIDE;

	/// <summary>From <b>DelegationController</b>: a control with the
	/// sub-controller-attribute has been created</summary>
	virtual IController* createSubController(UTF8StringPtr name,
		const IUIDescription* description, VST3Editor* editor) SMTG_OVERRIDE
	{return new MyUIController(editor);}

	/// <summary>From <b>IControlListener</b>: the value of a control
	/// changed</summary>
	virtual void valueChanged(CControl* pControl) SMTG_OVERRIDE;

private:
	/// <summary>our savestate and context</summary>
	EOSXsavestate saveState;

};

// **************************************************************************
}} // namespaces
