#include "QIPAddress.h"

namespace ShadowTec {

CQIPAddress::CQIPAddress()
{
}

CQIPAddress::~CQIPAddress()
{

}

// convert to string
void CQIPAddress::toString(std::string& rString) const
{
	char buffer[40];
	// check if ipv4 is sufficient
	if ((ippackA == IPV6TO4PACKA) && (ippackB == IPV6TO4PACKB) && (ippackC == IPV6TO4PACKC))
		snprintf(buffer, 40, "%d.%d.%d.%d",
		(0x0FF & (ippackD >> 24)), (0x0FF & (ippackD >> 16)),
		(0x0FF & (ippackD >> 8)), (0x0FF & ippackD)
		);
	else
		// write ipv6
		snprintf(buffer, 40, "%X:%X:%X:%X:%X:%X:%X:%X",
		(0x0FFFF & (ippackA >> 16)), (0x0FFFF & ippackA),
			(0x0FFFF & (ippackB >> 16)), (0x0FFFF & ippackB),
			(0x0FFFF & (ippackC >> 16)), (0x0FFFF & ippackC),
			(0x0FFFF & (ippackD >> 16)), (0x0FFFF & ippackD)
		);
	rString = buffer;
}


// convert to string
bool CQIPAddress::fromString(std::string& rString)
{
	int i = rString.length();
	bool isHex = false;
	bool isipv6 = false;
	bool insideDig = false;
	bool started = false;
	int blockend = i;
	__int32 blocks[10] = {0,0,0,0,0,0,0,0,0,0};
	int blkidx = 9;


	while (i-- > 0)
	{
		char c = rString[i];
		// skip tab and spaces
		if (c == ' ' || c == '\t')
			continue;
		if (c >= '0' && c <= '9')
		{
			if (!insideDig)
				blockend = i;
			insideDig = true;
			started = true;
		}
		// point seperates decimal blocks, but might be spam at end of string
		if (c == '.')
		{
			if (!started)
				continue;
			if (isHex)
				return false;
			if (!insideDig)
				break;
			// found decimal block
			blocks[blkidx] = atoi(rString.substr(i + 1, blockend - i + 1).c_str());
			insideDig = false;
			if (--blkidx < 0)
				break;
		}
	}

	if (insideDig)
	{
		blocks[blkidx] = atoi(rString.substr(i + 1, blockend - i + 1).c_str());
	}

	ippackD = (
		(0xFF & blocks[9]) | ((0xFF & blocks[8]) << 8) |
		((0xFF & blocks[7]) << 16) | ((0xFF & blocks[6]) << 24)
		);
	return true;
}

// set to ipv4-address
bool CQIPAddress::fromIPV4(__int32 packD)
{
	ippackA = IPV6TO4PACKA;
	ippackB = IPV6TO4PACKB;
	ippackC = IPV6TO4PACKC;
	ippackD = packD;
	return true;
}

}